% ficheiro de arranque de modelizacao Simulink para trajectorias 3D para o Nissan Leaf.
if ~exist(fullfile([cd filesep 'startFLmRB.m']),'file')
    error('The current folder should be ''FLmRB''.');
end
%----------------------
clc
clearvars -except dia10 dia13
clear global time_step batModel start_vel Leafphysics
global time_step batModel start_vel Leafphysics
%----------------------
disp('Verifying if the model is loaded in memory...');
try
    get_param('simulationModel','SimulationStatus');
    disp('Model is loaded');
catch
    disp('Model is not loaded, trying to load...');
    open_system('simulationModel');
    try
        get_param('simulationModel','SimulationStatus');
        disp('Model is now loaded');
    catch
        error('Could not load the model.');
    end;
end;
% get_param(0, 'CharacterEncoding')
% slCharacterEncoding('windows-1252');

batModel = 3;

%v----- Lookup tables -----v
% Leaf electrical motor efficiency
if exist(['dataSets' filesep 'EV_Data' filesep 'leafEfficiency.mat'], 'file') && ...
   exist(['dataSets' filesep 'EV_Data' filesep 'leafEfficiencyNeg.mat'], 'file')
        load(['dataSets' filesep 'EV_Data' filesep 'leafEfficiency.mat']);
        load(['dataSets' filesep 'EV_Data' filesep 'leafEfficiencyNeg.mat']);
else
    error('lookup table missing (leafEfficiency.mat or leafEfficiencyNeg.mat).');
end
if exist(['dataSets' filesep 'EV_Data' filesep 'SOC.mat'], 'file') &&...
   exist(['dataSets' filesep 'EV_Data' filesep 'Eoc.mat'], 'file') &&...
   exist(['dataSets' filesep 'EV_Data' filesep 'Rin.mat'], 'file')
        load(['dataSets' filesep 'EV_Data' filesep 'Rin.mat']);
        load(['dataSets' filesep 'EV_Data' filesep 'Eoc.mat']);
        load(['dataSets' filesep 'EV_Data' filesep 'SOC.mat']);
        resistance_profile = fliplr(Rin')';
        soc_profile = fliplr(SOCvet')';
        battery_profile = fliplr(Eoc2')';
        clear Rin SOCvet Eoc2
else
    error('lookup table missing (Rin.mat or Eoc.mat or SOC.mat).');
end
% Leaf electrical motor torque maximum as function of speed
if exist(['dataSets' filesep 'EV_Data' filesep 'leafTqMaxPos.mat'], 'file') && ...
   exist(['dataSets' filesep 'EV_Data' filesep 'leafTqMaxNeg.mat'], 'file')
        load(['dataSets' filesep 'EV_Data' filesep 'leafTqMaxPos.mat']);
        load(['dataSets' filesep 'EV_Data' filesep 'leafTqMaxNeg.mat']);
else
    error('lookup table missing (leafTqMaxPos.mat or leafTqMaxNeg.mat).');
end
%^----- Lookup tables -----^

%----------------------------------
%--------- Load data sets ---------
prompt = 'Select: 1-> Test#1;  2-> Test#2;  0-> cancel;  ';
option = input(char(prompt));
if option > 2
    option = 0;
end
if option == 0
    fprintf('cancelling...\n');
    datasize   = 0;
    Trial.signals.values = zeros(datasize,2);
    Trial.time = (0: 1 : datasize-1)';
    SOC        = [];
    time_step  = [];
    calc.I     = [];
    calc.V     = [];
    titulo     = [];
else
    fprintf('loading Test#%d ...\n', option);
    M = csvread(['dataSets/Test#' sprintf('%d',option) '.csv']);
    datasize = length(M);
    Trial.signals.values = zeros(datasize,2);
    Trial.time = (0: 1 : datasize-1)';
    Trial.signals.dimensions=2;
    Trial.signals.label='';
    Trial.blockName='';
    % load speed (m/s)
    Trial.signals.values(:,1) = M(:,1);
    % load inclination (rad)
    Trial.signals.values(:,2) = M(:,2);
    % load current (A)
    calc.I(:,1) = M(:,3);
    % load voltage (V)
    calc.V(:,1) = M(:,4);
    % load SOC (%);
    calc.SOC(:,1) = M(:,5);

    start_vel=Trial.signals.values(1,1);

    time_step = 0.1; % 100 ms.

    titulo = sprintf('Test#%d', option);
end
clear option M
%^-------- Load data sets --------^
%----------------------------------

if datasize ~= 0
    prompt='Use FLmRB? 1-> Yes;  2-> No  ';
    option = input(char(prompt));
    if option ~= 1
        option = 2;
    end
    withFIS = option == 1;
    clear option prompt
    fprintf('%s\n', 'Select folder to save files');
    dirToSave = uigetdir('', 'Select folder to save files...');
    if dirToSave == 0
        dirToSave = '';
    end
    [H.Y, H.M, H.D, H.H, H.MN, H.S] = datevec(now);
    dateStamp = sprintf('%02.0f-%02.0f-%02.0f_%02.0f-%02.0f-%02.0f',(H.Y/100-fix(H.Y/100))*100,H.M,H.D,H.H,H.MN,H.S);
    clear H

    carMass = 1521;
    gRatio  = 7.937;
    Leafphysics = ...
    struct('DELTA_T', time_step, ...
           'COEF_ATRITO_PNEU', 0.007,...
           'DENS_AR', 1.25, ...
           'COEF_ARRASTO', 0.28, ...
           'ACCEL_GRAV', 9.8, ...
           'CAPACITY', 65,...
           'PEUKERT_COEF_DEFAULT', 1,...
           'AREA_FRONTAL', 2.29, ...
           'WHEEL', 0.309, ...
           'GEAR_RATIO', gRatio, ...
           'POT_ACC', 269, ...
           'Mass', carMass ,...
           'passengers_mass', 280,...
           'CELL_NUMBER', 48,...
           'Inertial_mass', carMass * (0.04 + 0.0025 * gRatio .^2),...
           'Transm_Losses', 0.83,...
           'Bat_Losses', 0.97...
       );
    clear carMass gRatio

    fuzzyFiles1 = 'leaf_FLmRB_paper';
    fuzzyFiles2 = 'simulationModel';

    leaf_regen=readfis(fuzzyFiles1);

    set_param(fuzzyFiles2, 'StopTime', num2str(Trial.time(end,1)));

    fprintf('%s\n', 'simulation beginning');
    sim(fuzzyFiles2);
    fprintf('%s\n', 'end of simulation');

    fprintf('\n%s\n', 'some statistics:');
    % some statistics
    texto{1,1} = sprintf('Pot calc/real: %f', (sum(calc.I)/(datasize*time_step))/(sum(I_t)/(datasize*time_step)));
    r1 = sqrt(mse(I_t - calc.I)) / (max(I_t) - min(calc.I));
    texto{2,1} = sprintf('NRMSE: %f', r1);
    texto{3,1} = sprintf('---------------------------');
    texto{4,1} = sprintf('Pot(+) calc/real: %f', (sum(calc.I.*(calc.I>=0))/(datasize*time_step))/(sum(I_t.*(I_t>=0))/(datasize*time_step)));
    r2 = sqrt(mse(I_t.*(I_t>=0) - calc.I.*(I_t>=0))) / (max(I_t.*(I_t>=0)) - min(calc.I.*(I_t>=0)));
    texto{5,1} = sprintf('NRMSE: %f', r2);
    texto{6,1} = sprintf('Pot(-) calc/real: %f', (sum(calc.I.*(calc.I<0))/(datasize*time_step))/(sum(I_t.*(I_t<0))/(datasize*time_step)));
    r3 = sqrt(mse(I_t.*(I_t<0) - calc.I.*(I_t<0))) / (max(I_t.*(I_t<0)) - min(calc.I.*(I_t<0)));
    texto{7,1} = sprintf('NRMSE: %f', r3);
    disp(texto(:));
    % ---------------

    speed = Trial.signals.values(:,1);
    incls = Trial.signals.values(:,2);
    eixoX = cumtrapz(speed).*0.1.*0.001; % km

    % ---------------
    if exist('grafico', 'var')
        grafico = figure(grafico+1);hold on
    else
        grafico = figure;hold on
    end
    plot(eixoX(5:1:end),accel(5:1:end),'k','DisplayName','acel')
    plot(eixoX(5:1:end),(jerk(5:1:end)),'b','DisplayName','jerk')
    plot(eixoX(5:1:end),calc.I(5:1:end)/50,'g','DisplayName','I/50')
    plot(eixoX(5:1:end),I_t(5:1:end)/50,'r','DisplayName','I\_t/50');
    plot(eixoX(5:1:end),incls(5:1:end)*10,'m','DisplayName','incl*10');
    plot(eixoX(5:1:end),regFactor(5:1:end),'k','DisplayName','regFactor');
    title([titulo ' (nrmse:' num2str(r1) ' | ' num2str(r2) ' | ' num2str(r3)...
           '; Tr.Losses:' num2str(Leafphysics.Transm_Losses) ')']);
    xlim([0 100]);
    xlabel('distance [km]');
    legend show;
    grid on;
    % ---------------
    clear r1 r2 r3 grafico

    SL.fRR = fRR;SL.fAD = fAD;SL.fHC = fHC;SL.fWA = fWA;SL.fLA = fLA;SL.fMB = fMB;
    clear fRR fAD fHC fWA fLA fMB
    if ~strcmp(dirToSave, '')
        save([dirToSave filesep regexprep(titulo,'[^\w-'']','') '_' dateStamp '.mat'],...
             'jerk', 'accel', 'calc', 'I_t', 'incls', 'regFactor', 'Vt', 'soc_up',...
             'speed', 'Vel_angular', 'batModel', 'E_OCV', 'Rbat', 'withFIS', ...
             'SL', 'total_force', 'tq_req', 'Power_req_total', ...
             'rendimento', 'Tq_Over_Neg', 'Tq_Over_Pos', 'motor_tq', ...
             'titulo', 'Leafphysics', 'texto', 'leaf_regen', ...
             'eixoX', 'dateStamp', 'brk_tq', 'brk_tq_over');
    end
    fprintf('%s\n', 'program finished');
else
    fprintf('%s\n', 'program aborted');
end

clear battery_profile datasize dirToSave executado fuzzyFiles1 fuzzyFiles2 grafico leafEfficiency ...
      leafEfficiencyNeg leafTqMaxNeg leafTqMaxPos resistance_profile ...
      soc_profile sumoRoot Trial tElapsed tStart tout X Y
